@@ -29,8 +29,6 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id$ */
-
 /* avr/iopwm1.h - definitions for AT90PWM1 device */
 
 #ifndef _AVR_IOPWM1_H_
@@ -1156,4 +1154,8 @@
 #define SLEEP_MODE_STANDBY (0x06<<1)
 
 
+#define SIGNATURE_0 0x1E
+#define SIGNATURE_1 0x93
+#define SIGNATURE_2 0x83
+
 #endif /* _AVR_IOPWM1_H_ */
